/*

* Copyright (c) 2003-2006 Illinois Institute of Technology.
* All rights reserved.
*
* This file is part of the NBP software package. For license
* information, see the LICENSE file in the top level directory of the
* NBP source distribution.
*
* Released Date: September 5 2006
* This is the NBP I release by the
* NBP Group,
* http://www.cs.iit.edu/~nbp
* Department of Computer Science,
* Illinois Institute of Technology.
*
* This release has been tested under Windows XP Professional
*
* Supervisor
* ----------
* Illiniois Institute of Technology:
* -Dr. Xian-He Sun
*
* Author(s)
* -----------------
* Illinois Institute of Technology:
* -Alaknantha Eswaradass
* -Ming Wu
* -Xian-He Sun
*
*/
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


/**
*
 * Neural Network configuration panel
 **/
public class NNPanel extends JPanel{
	
	private JPanel parmPanel = new JPanel();
	private JPanel forcastPanel = new JPanel();
	private JLabel lblLearningRate = new JLabel("Learning Rate");
	private JLabel lblEpochs = new JLabel("Max. no. of epochs");
	private JLabel lblLayers = new JLabel("No. of hidden Layers");
	
	private JTextField txtLearningRate= new JTextField();
	private JTextField txtEpochs= new JTextField();
	private JTextField txtLayers= new JTextField();
	
	private JButton btnSteps  = new JButton("Show steps");
	private JLabel lblStep = new JLabel("Prediction Step");
	private JLabel lblBin  = new JLabel("Bin Size");
	private JLabel lblTo = new JLabel("    to   ");
	private JLabel lblTo2 = new JLabel("   to    ");
	private JLabel lblInSteps= new JLabel("in steps of");
	private JLabel lblInSteps2= new JLabel("in steps of");
	
	private JTextField txtStepFrom = new JTextField();
	private JTextField txtStepStep = new JTextField();
	private JTextField txtStepTo = new JTextField();
	private JTextField txtBinFrom = new JTextField();
	private JTextField txtBinTo = new JTextField();
	private JTextField txtStepsOf = new JTextField();
	
	private TitledBorder forcastBorder = null;
	public NNPanel(){
		TitledBorder paramBorder = BorderFactory.createTitledBorder("ANN Parameter");
		Dimension dim = new Dimension(100,20);
		parmPanel.setBorder(paramBorder);
		parmPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;
        c.gridwidth = 1;
        c.gridheight= 1;
        c.insets=new Insets(10, 10, 10, 10);
        c.gridx = 0; c.gridy=0;
		parmPanel.add(lblLearningRate,c);
		c.gridx = 1; c.gridy=0;
		parmPanel.add(txtLearningRate,c);
		txtLearningRate.setPreferredSize(dim);
		txtLearningRate.setMaximumSize(dim);
		txtLearningRate.setMaximumSize(dim);
		parmPanel.revalidate();
		
		c.gridx = 0; c.gridy=1;
		parmPanel.add(lblEpochs,c);
		c.gridx = 1; c.gridy=1;
		parmPanel.add(txtEpochs,c);
		txtEpochs.setPreferredSize(dim);
		txtEpochs.setMaximumSize(dim);
		txtEpochs.setMaximumSize(dim);
		c.gridx = 0; c.gridy=2;
		parmPanel.add(lblLayers,c);
		c.gridx = 1; c.gridy=2;
		parmPanel.add(txtLayers,c);
		txtLayers.setPreferredSize(dim);
		txtLayers.setMaximumSize(dim);
		txtLayers.setMaximumSize(dim);
		parmPanel.setPreferredSize(new Dimension(200,100));
		
		forcastBorder = BorderFactory.createTitledBorder("Forecast Inputs");
		forcastPanel.setBorder(forcastBorder);
		forcastPanel.setLayout(new GridBagLayout());
        c.insets=new Insets(2, 2, 2, 2);
		c.gridx = 0; c.gridy=0;
		forcastPanel.add(lblStep,c);
		c.gridx = 1; c.gridy=0;
		forcastPanel.add(txtStepFrom,c);
		c.gridx=2; c.gridy=0;
		c.gridwidth=2;
		forcastPanel.add(btnSteps,c);
		btnSteps.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				displaySteps();
			}
		});
		c.gridwidth=1;
		c.gridx = 0; c.gridy=1;
		forcastPanel.add(lblBin,c);
		c.gridx = 1; c.gridy=1;
		forcastPanel.add(txtBinFrom,c);
		
		
		forcastPanel.revalidate();
		
		setLayout(new BorderLayout());
		add(parmPanel,BorderLayout.CENTER);
		add(forcastPanel,BorderLayout.SOUTH);
		this.revalidate();
		setPreferredSize(new Dimension(300,380));
		
	}
	
	private void displaySteps(){
		forcastPanel.remove(btnSteps);
		GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;
        c.gridwidth = 1;
        c.gridheight= 1;
		c.insets=new Insets(2, 2, 2, 2);
		
		c.gridx = 2; c.gridy=0;
		forcastPanel.add(lblTo,c);
		c.gridx = 3; c.gridy=0;
		forcastPanel.add(txtStepTo,c);
		c.gridx = 4; c.gridy=0;
		forcastPanel.add(lblInSteps,c);
		c.gridx = 5; c.gridy=0;
		forcastPanel.add(txtStepStep,c);
		
		c.gridx = 2; c.gridy=1;
		forcastPanel.add(lblTo2,c);
		c.gridx = 3; c.gridy=1;
		forcastPanel.add(txtBinTo,c);
		c.gridx = 4; c.gridy=1;
		forcastPanel.add(lblInSteps2,c);
		c.gridx = 5; c.gridy=1;
		forcastPanel.add(txtStepsOf,c);
		forcastPanel.revalidate();
	}
	
	
	public Config getConfig(){
		Config config = new Config();
		config.nnConfig = new NNConfig();
		try{
			config.nnConfig.learningRate = Float.parseFloat(txtLearningRate.getText());
		}catch(NumberFormatException nfe){
			config.nnConfig.learningRate = GUI.DefaultConfig.nnConfig.learningRate;
		}
		try{
			config.nnConfig.maxOfEpochs = Integer.parseInt(txtEpochs.getText());
		}catch(NumberFormatException nfe){
			config.nnConfig.maxOfEpochs = GUI.DefaultConfig.nnConfig.maxOfEpochs;
		}
		try{
			config.nnConfig.noOFHiddenLayers = Integer.parseInt(txtLayers.getText());
		}catch(NumberFormatException nfe){
			config.nnConfig.noOFHiddenLayers = GUI.DefaultConfig.nnConfig.noOFHiddenLayers;
		}
		config.fiConfig = new FIConfig();
		try{
			config.fiConfig.predictionStepFrom = Integer.parseInt(txtStepFrom.getText());
		}catch(NumberFormatException nfe){
			config.fiConfig.predictionStepFrom = GUI.DefaultConfig.fiConfig.predictionStepFrom;
		}
		try{
			config.fiConfig.predictionStepTo = Integer.parseInt(txtStepTo.getText());
		}catch(NumberFormatException nfe){
			config.fiConfig.predictionStepTo = GUI.DefaultConfig.fiConfig.predictionStepTo;
		}
		try{
			config.fiConfig.predictionStepStep = Integer.parseInt(txtStepStep.getText());
		}catch(NumberFormatException nfe){
			config.fiConfig.predictionStepStep = GUI.DefaultConfig.fiConfig.predictionStepStep;
		}
		try{
			config.fiConfig.binSizeFrom = Integer.parseInt(txtBinFrom.getText());
		}catch(NumberFormatException nfe){
			config.fiConfig.binSizeFrom = GUI.DefaultConfig.fiConfig.binSizeFrom;
		}
		try{
			config.fiConfig.binSizeTo = Integer.parseInt(txtBinTo.getText());
		}catch(NumberFormatException nfe){
			config.fiConfig.binSizeTo = GUI.DefaultConfig.fiConfig.binSizeTo;
		}
		try{
			config.fiConfig.binSizeStep = Integer.parseInt(txtStepsOf.getText());		
		}catch(NumberFormatException nfe){
			config.fiConfig.binSizeStep = GUI.DefaultConfig.fiConfig.binSizeStep;
		}
		return config;
	}
	
	public void setConfig(Config config){
		txtLearningRate.setText(Float.toString(config.nnConfig.learningRate));
		txtEpochs.setText(Integer.toString(config.nnConfig.maxOfEpochs));
		txtLayers.setText(Integer.toString(config.nnConfig.noOFHiddenLayers));
		
		txtStepFrom.setText(Integer.toString(config.fiConfig.predictionStepFrom));
		txtStepStep.setText(Integer.toString(config.fiConfig.predictionStepStep));
		txtStepTo.setText(Integer.toString(config.fiConfig.predictionStepTo));
		txtBinFrom.setText(Integer.toString(config.fiConfig.binSizeFrom));
		txtBinTo.setText(Integer.toString(config.fiConfig.binSizeTo));
		txtStepsOf.setText(Integer.toString(config.fiConfig.binSizeStep));
	}
	
	
	 public static void main(String argv[]){
		   JFrame frame = new JFrame("NN Test");
		   frame.setVisible(true);
		   frame.setSize(500, 500);
		  
		   frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		   NNPanel nnPanel = new NNPanel();
		   frame.getContentPane().setLayout(new BorderLayout());
		   frame.getContentPane().add(nnPanel,BorderLayout.CENTER);
		   frame.validate();
	   }	
}
